"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportExportSpecifierReplacer = void 0;
const tslib_1 = require("tslib");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
class ImportExportSpecifierReplacer extends typescript_transformer_common_1.NodeReplacer {
    replacer;
    kinds = [
        typescript_1.default.SyntaxKind.ImportDeclaration,
        typescript_1.default.SyntaxKind.ExportDeclaration,
        typescript_1.default.SyntaxKind.CallExpression,
    ];
    constructor(replacer) {
        super();
        this.replacer = replacer;
    }
    check(node, logger) {
        try {
            // console.log(' ! ', ts.SyntaxKind[node.kind]);
            if (!(0, typescript_transformer_common_1.isImportExportFrom)(node)) {
                if (typescript_1.default.isCallExpression(node)) {
                    if (node.parent && !typescript_1.default.isSourceFile(node.parent)) {
                        return false;
                    }
                    if (typescript_1.default.isIdentifier(node.expression)) {
                        if (node.expression.escapedText === 'require' &&
                            node.arguments.length === 1 &&
                            typescript_1.default.isStringLiteral(node.arguments[0])) {
                            return true;
                        }
                    }
                }
                return false;
            }
        }
        catch (e) {
            logger.error(e.message);
            return false;
        }
        return true;
    }
    createString(node, context) {
        const str = this.replacer(node.moduleSpecifier.text, node, context, this.logger);
        if (node.moduleSpecifier.text === str) {
            return undefined;
        }
        return context.factory.createStringLiteral(str);
    }
    _replace(node, context) {
        // console.debug(' * %s', node.moduleSpecifier.text);
        if (typescript_1.default.isImportDeclaration(node)) {
            const moduleSpecifier = this.createString(node, context);
            if (!moduleSpecifier)
                return node;
            return context.factory.updateImportDeclaration(node, node.decorators, node.modifiers, node.importClause, moduleSpecifier, node.assertClause);
        }
        else if (typescript_1.default.isExportDeclaration(node)) {
            const moduleSpecifier = this.createString(node, context);
            if (!moduleSpecifier)
                return node;
            return context.factory.updateExportDeclaration(node, node.decorators, node.modifiers, node.isTypeOnly, node.exportClause, moduleSpecifier, node.assertClause);
        }
        else {
            // require("xxx")
            const sl = node.arguments[0];
            (0, typescript_transformer_common_1.linkParentNode)(node, typescript_1.default.getOriginalNode(sl).parent);
            const str = this.replacer(sl.text, node, context, this.logger);
            if (sl.text === str) {
                return node;
            }
            const newStr = context.factory.createStringLiteral(str);
            typescript_1.default.setOriginalNode(newStr, typescript_1.default.getOriginalNode(sl));
            return context.factory.updateCallExpression(node, node.expression, node.typeArguments, [newStr]);
        }
    }
}
exports.ImportExportSpecifierReplacer = ImportExportSpecifierReplacer;
//# sourceMappingURL=importSpecifier.cjs.map
