import _build_script_typescript_transformer_common_0 from '@build-script/typescript-transformer-common';
const { IDebug, isImportExportFrom, linkParentNode, NodeReplacer, ValidImportOrExportFromDeclaration } = _build_script_typescript_transformer_common_0;
import ts from 'typescript';
export class ImportExportSpecifierReplacer extends NodeReplacer {
    replacer;
    kinds = [
        ts.SyntaxKind.ImportDeclaration,
        ts.SyntaxKind.ExportDeclaration,
        ts.SyntaxKind.CallExpression,
    ];
    constructor(replacer) {
        super();
        this.replacer = replacer;
    }
    check(node, logger) {
        try {
            // console.log(' ! ', ts.SyntaxKind[node.kind]);
            if (!isImportExportFrom(node)) {
                if (ts.isCallExpression(node)) {
                    if (node.parent && !ts.isSourceFile(node.parent)) {
                        return false;
                    }
                    if (ts.isIdentifier(node.expression)) {
                        if (node.expression.escapedText === 'require' &&
                            node.arguments.length === 1 &&
                            ts.isStringLiteral(node.arguments[0])) {
                            return true;
                        }
                    }
                }
                return false;
            }
        }
        catch (e) {
            logger.error(e.message);
            return false;
        }
        return true;
    }
    createString(node, context) {
        const str = this.replacer(node.moduleSpecifier.text, node, context, this.logger);
        if (node.moduleSpecifier.text === str) {
            return undefined;
        }
        return context.factory.createStringLiteral(str);
    }
    _replace(node, context) {
        // console.debug(' * %s', node.moduleSpecifier.text);
        if (ts.isImportDeclaration(node)) {
            const moduleSpecifier = this.createString(node, context);
            if (!moduleSpecifier)
                return node;
            return context.factory.updateImportDeclaration(node, node.decorators, node.modifiers, node.importClause, moduleSpecifier, node.assertClause);
        }
        else if (ts.isExportDeclaration(node)) {
            const moduleSpecifier = this.createString(node, context);
            if (!moduleSpecifier)
                return node;
            return context.factory.updateExportDeclaration(node, node.decorators, node.modifiers, node.isTypeOnly, node.exportClause, moduleSpecifier, node.assertClause);
        }
        else {
            // require("xxx")
            const sl = node.arguments[0];
            linkParentNode(node, ts.getOriginalNode(sl).parent);
            const str = this.replacer(sl.text, node, context, this.logger);
            if (sl.text === str) {
                return node;
            }
            const newStr = context.factory.createStringLiteral(str);
            ts.setOriginalNode(newStr, ts.getOriginalNode(sl));
            return context.factory.updateCallExpression(node, node.expression, node.typeArguments, [newStr]);
        }
    }
}
//# sourceMappingURL=importSpecifier.js.map